package com.tzld.piaoquan.recommend.server.service.filter.strategy;

import com.google.common.base.Stopwatch;
import com.tzld.piaoquan.recommend.server.common.ThreadPoolFactory;
import com.tzld.piaoquan.recommend.server.repository.VideoAppTypeStatus;
import com.tzld.piaoquan.recommend.server.repository.VideoAppTypeStatusRepository;
import com.tzld.piaoquan.recommend.server.repository.WxVideoStatus;
import com.tzld.piaoquan.recommend.server.repository.WxVideoStatusRepository;
import com.tzld.piaoquan.recommend.server.service.filter.FilterParam;
import com.tzld.piaoquan.recommend.server.service.filter.FilterStrategy;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.RandomUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.dao.DataAccessException;
import org.springframework.data.redis.core.RedisOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.SessionCallback;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author dyp
 */
@Component
@Slf4j
public class RecommendStatusStrategy implements FilterStrategy {
    @Autowired
    @Qualifier("redisTemplate")
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    @Qualifier("videoRedisTemplate")
    private RedisTemplate<String, String> videoRedisTemplate;

    @Autowired
    private WxVideoStatusRepository wxVideoStatusRepository;

    @Autowired
    private VideoAppTypeStatusRepository videoAppTypeStatusRepository;

    private final String keyFormat = "video:recommend:status:%s";

    private final String videoAppTypeStatusKeyFormat = "video:active:status:%s:%s";

    private static final int RECOMMEND_STATUS = -6;

    @Override
    public List<Long> filter(FilterParam param) {
        if (param == null
                || CollectionUtils.isEmpty(param.getVideoIds())) {
            return Collections.emptyList();
        }

        List<String> keys = param.getVideoIds().stream()
                .map(id -> String.format(keyFormat, id))
                .collect(Collectors.toList());

        List<String> recommendStatus = redisTemplate.opsForValue().multiGet(keys);
        List<Long> cacheMissVideoIds = new ArrayList<>();
        Map<Long, Integer> recommendStatusMap = new HashMap<>();
        for (int i = 0; i < param.getVideoIds().size(); i++) {
            String value = recommendStatus.get(i);
            if (StringUtils.isBlank(value)) {
                cacheMissVideoIds.add(param.getVideoIds().get(i));
            } else {
                recommendStatusMap.put(param.getVideoIds().get(i), NumberUtils.toInt(value));
            }
        }

        if (CollectionUtils.isNotEmpty(cacheMissVideoIds)) {
            List<WxVideoStatus> status = wxVideoStatusRepository.findAllByVideoIdIn(cacheMissVideoIds);
            if (CollectionUtils.isNotEmpty(status)) {
                for (WxVideoStatus v : status) {
                    recommendStatusMap.put(v.getVideoId(), v.getRecommendStatus());

                    // TODO 异步更新缓存
                    ThreadPoolFactory.defaultPool().execute(() -> {
                        redisTemplate.executePipelined(new SessionCallback<String>() {
                            @Override
                            public <A, B> String execute(RedisOperations<A, B> redisOperations) throws DataAccessException {
                                ValueOperations<String, String> operations =
                                        (ValueOperations<String, String>) redisOperations.opsForValue();
                                status.forEach(v -> {
                                    operations.set(String.format(keyFormat, v.getVideoId()),
                                            String.valueOf(v.getRecommendStatus()), RandomUtils.nextInt(30, 60),
                                            TimeUnit.SECONDS);
                                });

                                return null;
                            }
                        });
                    });
                }
            }
        }

        List<Long> filterAfterRecommendStatus = param.getVideoIds().stream()
                .filter(id -> recommendStatusMap.containsKey(id) && recommendStatusMap.get(id) == RECOMMEND_STATUS)
                .collect(Collectors.toList());

        filterActiveStatusVideoId(filterAfterRecommendStatus, param.getAppType());

        return filterAfterRecommendStatus;

    }

    /**
     * 过滤上下架状态
     * <p>
     * 紧急需求上线
     * 性能待优化
     *
     * @param idList  videoId列表
     * @param appType appType
     */
    private void filterActiveStatusVideoId(List<Long> idList, Integer appType) {
        Stopwatch stopwatch = Stopwatch.createStarted();
        if (CollectionUtils.isEmpty(idList)) {
            return;
        }
        Iterator<Long> iterator = idList.iterator();
        while (iterator.hasNext()) {
            Long videoId = iterator.next();
            if (Objects.isNull(videoId)) {
                iterator.remove();
                continue;
            }
            String activeStatus = videoRedisTemplate.opsForValue().get(String.format(videoAppTypeStatusKeyFormat, appType, videoId));
            if (Objects.isNull(activeStatus)) {
                //查库
                List<VideoAppTypeStatus> videoAppTypeVideoStatusList =
                        videoAppTypeStatusRepository.findAllByVideoIdAndAppType(videoId, appType);
                if (Objects.isNull(videoAppTypeVideoStatusList) || videoAppTypeVideoStatusList.isEmpty()) {
                    //无数据 刷1 进redis
                    videoRedisTemplate.opsForValue().set(String.format(videoAppTypeStatusKeyFormat, appType, videoId), "1"
                            , 15, TimeUnit.DAYS);
                    activeStatus = "1";
                } else {
                    VideoAppTypeStatus videoAppTypeStatus = videoAppTypeVideoStatusList.get(0);
                    if (Objects.nonNull(videoAppTypeStatus) && Objects.nonNull(videoAppTypeStatus.getVideoStatus())) {
                        String videoStatus = videoAppTypeStatus.getVideoStatus().toString();
                        videoRedisTemplate.opsForValue().set(String.format(videoAppTypeStatusKeyFormat, appType, videoId), videoStatus
                                , 15, TimeUnit.DAYS);
                        activeStatus = videoStatus;
                    }
                }
            }
            if (Objects.equals("0", activeStatus)) {
                iterator.remove();
            }
        }
    }

}
